/*
 *      uywane peryferia
 *      usart0 - dmx
 *      usart1 - komunikacja bluetooth z apk
 *      timer0 - 8bit - zabezpieczenie poprawnosci interpretacji danych odbieranych przez bluetooth
 *      timer1 - 16bit - 40Hz, wysyanie pakietw DMX
 *      timer3 - 16bit - odliczanie czasu zaczenia dymiarki
 */

#define PORT(x) XPORT(x)
#define XPORT(x) (PORT##x)
#define PIN(x) XPIN(x)
#define XPIN(x) (PIN##x)
#define DDR(x) XDDR(x)
#define XDDR(x) (DDR##x)

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define DMX_SIZE		512
#define DMX_TX_PORT		E
#define DMX_TX_PIN_NMB	1

//ustawienie adresw urzdze odbiorczych (0x000 - 0x1FF)
#define DMX_ADDRESS_RGB			0x010
#define DMX_ADDRESS_LISTWY		0x020
#define DMX_ADDRESS_DYMIARKA	0x030
#define DMX_ADDRESS_PULSATOR	0x040

volatile uint8_t dmx_to_send[ DMX_SIZE ];	//globalna - zainicjalizowana zerami
volatile uint8_t dmx_buf[ DMX_SIZE ];
volatile uint16_t cnt_dmx;
volatile uint8_t flaga_40Hz;

void usart0_init();
void usart0_off();
void send_dmx ();

//dla bluetooth
volatile uint8_t parametr;
volatile uint8_t argument;
volatile uint8_t flaga_odbioru_pakietu_blue;
volatile uint8_t cnt_uart1 = 0;
#define TIMER0_5MS_START	TCCR0 |=  (1<<CS02)|(1<<CS01)|(1<<CS00)		//preskaler = 1024
#define TIMER0_5MS_STOP		TCCR0 &= ~((1<<CS02)|(1<<CS01)|(1<<CS00))

typedef struct {
	uint8_t wl_wyl_glowny;
	uint8_t podbicie;
	uint8_t wygaszenie;
	uint8_t en_out;
} Glowne_parametry_Init_Structure;
volatile Glowne_parametry_Init_Structure glowne_parametry;

typedef struct {
	uint8_t wl_wyl_rgb;
	uint8_t tryb;
	uint8_t sekwencja;
	uint8_t tempo;
	uint8_t blokowanie_tempa;
	uint8_t sterowanie_dzwiekiem;
	uint8_t tempo_default;
	uint8_t jasnosc;
	uint8_t tryb_manual;
	uint8_t red;
	uint8_t green;
	uint8_t blue;
} Reflektory_RGB_parametry_Init_Structure;
volatile Reflektory_RGB_parametry_Init_Structure reflektory_rgb_parametry;

typedef struct {
	uint8_t wl_wyl_listwy;
	uint8_t tryb;
	uint8_t tryb_manual;
	uint8_t dimmer;
} Listwy_parametry_Init_Structure;
volatile Listwy_parametry_Init_Structure listwy_parametry;

typedef struct {
	uint8_t wl_wyl_pulsator;
	uint8_t tryb;
	uint8_t tryb_manual;
	uint8_t obrot;
	uint8_t red;
	uint8_t green;
	uint8_t blue;
} Pulsator_RGB_parametry_Init_Structure;
volatile Pulsator_RGB_parametry_Init_Structure pulsator_rgb_parametry;

typedef struct {
	uint8_t czas_zalaczenia;
	uint8_t en_dym;
	uint8_t wl_dym;
} Dymiarka_parametry_Init_Structure;
volatile Dymiarka_parametry_Init_Structure dymiarka_parametry;

//dymiarka
volatile uint8_t flaga_timer_dym;	//gdy dymiarka jest zaczona na okreslony czas ta flaga jest ustawiana, kasowana gdy minie czas zalaczenia
									//wczesniej musi byc aktywne EN_DYM, a zalaczenie nastepuje po wcisnieciu EN_OUT
									//flaga jest potrzebna do blokowania wyczenia dymiarki, jesli przycisk SW_DYM nie jest aktywny
#define TIMER3_1s_START		TCCR3B |=   (1<<CS32)|(1<<CS30);	//preskaler - 1024
#define TIMER3_1s_STOP		TCCR3B &= ~((1<<CS32)|(1<<CS30));
volatile uint8_t czas_zalaczenia_dymiarki_buf;	//aby po wlaczeniu nie zmienil sie czas zalaczenia, gdy bedziemy przestawiac w apce

void usart1_init();	//dla odbioru bluetooth


//volatile uint8_t q;
//volatile uint8_t flaga_timer2;

int main(void) {

	reflektory_rgb_parametry.tryb = 1;
	reflektory_rgb_parametry.tempo = 90;
	reflektory_rgb_parametry.jasnosc = 4;
	listwy_parametry.wl_wyl_listwy = 1;
	listwy_parametry.tryb = 1;
	pulsator_rgb_parametry.tryb = 1;
	dymiarka_parametry.czas_zalaczenia = 1;

	//Timer1 do ustawiania flagi dla wysyania pakietow DMX//////////////////////////
	TCCR1B |= (1<<CS11)|(1<<CS10);		//preskaler = 64
	TCCR1B |= (1<<WGM12);				//tryb CTC
	TIMSK |= (1<<OCIE1A);				//zezw na przerw
	OCR1A = 6249;						//40Hz - 6249
	/////////////////////max 44.13 teoretycznie, prakt. wartosc 6102 czyli 40,96Hz
	//////////////////////////////////////////////////////////////////////////////////

	usart0_init();	//dmx
	usart1_init();	//bluetooth

	//Timer0 do zerowania licznika odbioru przez bluetooth dla uart1 po danym czasie, gdy nic nie jest nadawane
	TCCR0 |= (1<<WGM01);
	TIMSK |= (1<<OCIE0);
	OCR0 = 78;				//okolo 5 ms, bo preskaler 1024

	//Timer3 do odliczania czasu zaczenia dymiarki
										//preskaler = 1024
	TCCR3B |= (1<<WGM32);				//tryb CTC
	ETIMSK |= (1<<OCIE3A);				//zezw na przerw
	OCR3A = 15625;						//1Hz

	sei();

	DDRB |= (1<<PB5);

	///////////////////////////////testend////////////////////////////////

	_delay_ms(10);

	uint8_t tryb_manual_pulsator_rgb_pomocniczy = 0;
	uint8_t flaga_wygaszenie_pulsator = 0;

	while(1) {

		if(flaga_40Hz) {
			PORTB ^= (1<<PB5);
			flaga_40Hz = 0;
			send_dmx();
		}

		if(flaga_odbioru_pakietu_blue == 1) {
			switch(parametr) {
			case 1:
				glowne_parametry.wl_wyl_glowny = argument;
				break;
			case 2:
				glowne_parametry.podbicie = argument;
				break;
			case 3:
				glowne_parametry.wygaszenie = argument;
				break;
			case 4:
				glowne_parametry.en_out = argument;
				break;
			case 10:
				reflektory_rgb_parametry.wl_wyl_rgb = argument;
				break;
			case 11:
				reflektory_rgb_parametry.tryb = argument;
				break;
			case 12:
				reflektory_rgb_parametry.sekwencja = argument;
				break;
			case 13:
				reflektory_rgb_parametry.tempo = argument;
				break;
			case 14:
				reflektory_rgb_parametry.blokowanie_tempa = argument;
				break;
			case 15:
				reflektory_rgb_parametry.sterowanie_dzwiekiem = argument;
				break;
			case 16:
				reflektory_rgb_parametry.tempo_default = argument;
				break;
			case 17:
				reflektory_rgb_parametry.jasnosc = argument;
				break;
			case 18:
				reflektory_rgb_parametry.tryb_manual = argument;
				break;
			case 19:
				reflektory_rgb_parametry.red = argument;
				break;
			case 20:
				reflektory_rgb_parametry.green = argument;
				break;
			case 21:
				reflektory_rgb_parametry.blue = argument;
				break;
			case 30:
				listwy_parametry.wl_wyl_listwy = argument;
				break;
			case 31:
				listwy_parametry.tryb = argument;
				break;
			case 32:
				listwy_parametry.tryb_manual = argument;
				break;
			case 33:
				listwy_parametry.dimmer = argument;
				break;
			case 40:
				pulsator_rgb_parametry.wl_wyl_pulsator = argument;
				break;
			case 41:
				pulsator_rgb_parametry.tryb = argument;
				break;
			case 42:
				pulsator_rgb_parametry.tryb_manual = argument;
				break;
			case 43:
				pulsator_rgb_parametry.obrot = argument;
				break;
			case 44:
				pulsator_rgb_parametry.red = argument;
				break;
			case 45:
				pulsator_rgb_parametry.green = argument;
				break;
			case 46:
				pulsator_rgb_parametry.blue = argument;
				break;
			case 50:
				dymiarka_parametry.czas_zalaczenia = argument;
				break;
			case 51:
				dymiarka_parametry.en_dym = argument;
				break;
			case 52:
				dymiarka_parametry.wl_dym = argument;
				break;
			}
			flaga_odbioru_pakietu_blue = 0;
		}

		if(!glowne_parametry.wl_wyl_glowny) {
			dmx_buf[DMX_ADDRESS_RGB] = 0;
			dmx_buf[DMX_ADDRESS_PULSATOR] = 0;
			dmx_buf[DMX_ADDRESS_PULSATOR+1] = 0;
			dmx_buf[DMX_ADDRESS_PULSATOR+2] = 0;
			dmx_buf[DMX_ADDRESS_PULSATOR+3] = 0;
			dmx_buf[DMX_ADDRESS_PULSATOR+4] = 0;
			tryb_manual_pulsator_rgb_pomocniczy = 0;
//			dmx_buf[DMX_ADDRESS_DYMIARKA] = 0;
		}
		if(!reflektory_rgb_parametry.wl_wyl_rgb) dmx_buf[DMX_ADDRESS_RGB] = 0;
		if((!pulsator_rgb_parametry.wl_wyl_pulsator) || glowne_parametry.wygaszenie) {
			dmx_buf[DMX_ADDRESS_PULSATOR] = 0;
			dmx_buf[DMX_ADDRESS_PULSATOR+1] = 0;
			dmx_buf[DMX_ADDRESS_PULSATOR+2] = 0;
			dmx_buf[DMX_ADDRESS_PULSATOR+3] = 0;
			dmx_buf[DMX_ADDRESS_PULSATOR+4] = 0;
		}

//		dmx_buf[DMX_ADDRESS_LISTWY+2] = (dmx_buf[DMX_ADDRESS_LISTWY+2] & 0xFE) | (listwy_parametry.wl_wyl_listwy << 0);
		dmx_buf[DMX_ADDRESS_RGB + 3] = (dmx_buf[DMX_ADDRESS_RGB + 3] & 0xFD) | (glowne_parametry.podbicie << 1);
		dmx_buf[DMX_ADDRESS_LISTWY+2] = (dmx_buf[DMX_ADDRESS_LISTWY+2] & 0xFD) | (glowne_parametry.podbicie << 1);
		dmx_buf[DMX_ADDRESS_RGB + 3] = (dmx_buf[DMX_ADDRESS_RGB + 3] & 0xFE) | (glowne_parametry.wygaszenie << 0);
		dmx_buf[DMX_ADDRESS_LISTWY+2] = (dmx_buf[DMX_ADDRESS_LISTWY+2] & 0xFB) | (glowne_parametry.wygaszenie << 2);


		dmx_buf[DMX_ADDRESS_RGB + 5] = reflektory_rgb_parametry.red;
		dmx_buf[DMX_ADDRESS_RGB + 6] = reflektory_rgb_parametry.green;
		dmx_buf[DMX_ADDRESS_RGB + 7] = reflektory_rgb_parametry.blue;


		dmx_buf[DMX_ADDRESS_LISTWY + 1] = listwy_parametry.dimmer;

//		dmx_buf[DMX_ADDRESS_LISTWY + 2] = (dmx_buf[DMX_ADDRESS_LISTWY + 2] & 0xFE) | ((listwy_parametry.wl_wyl_listwy | (!glowne_parametry.wl_wyl_glowny)) << 0);

		if( listwy_parametry.wl_wyl_listwy | (!glowne_parametry.wl_wyl_glowny) )	dmx_buf[DMX_ADDRESS_LISTWY + 2] |= 0x01;	//ustaw flage wylaczenia

		if(glowne_parametry.wygaszenie) flaga_wygaszenie_pulsator = 1;
		if(pulsator_rgb_parametry.wl_wyl_pulsator && glowne_parametry.wl_wyl_glowny && tryb_manual_pulsator_rgb_pomocniczy && (!glowne_parametry.wygaszenie)) {
			dmx_buf[DMX_ADDRESS_PULSATOR + 1] = pulsator_rgb_parametry.red;
			dmx_buf[DMX_ADDRESS_PULSATOR + 2] = pulsator_rgb_parametry.green;
			dmx_buf[DMX_ADDRESS_PULSATOR + 3] = pulsator_rgb_parametry.blue;
		}
		if(dymiarka_parametry.wl_dym && glowne_parametry.wl_wyl_glowny) dmx_buf[DMX_ADDRESS_DYMIARKA] = 200;
		else if(!flaga_timer_dym) dmx_buf[DMX_ADDRESS_DYMIARKA] = 0;

		if(glowne_parametry.en_out) {

			//-----reflektory----RGB-------------
			if(reflektory_rgb_parametry.wl_wyl_rgb && glowne_parametry.wl_wyl_glowny) {
				if(reflektory_rgb_parametry.tryb_manual) dmx_buf[DMX_ADDRESS_RGB] = 9;
				else dmx_buf[DMX_ADDRESS_RGB] = reflektory_rgb_parametry.tryb;
			}
			dmx_buf[DMX_ADDRESS_RGB + 1] = reflektory_rgb_parametry.sekwencja;
			dmx_buf[DMX_ADDRESS_RGB + 2] = reflektory_rgb_parametry.tempo;
			dmx_buf[DMX_ADDRESS_RGB + 3] = (dmx_buf[DMX_ADDRESS_RGB + 3] & 0xE3) |
					(reflektory_rgb_parametry.blokowanie_tempa << 4 ) |
					(reflektory_rgb_parametry.sterowanie_dzwiekiem << 3 ) |
					(reflektory_rgb_parametry.tempo_default << 2);
			dmx_buf[DMX_ADDRESS_RGB + 4] = reflektory_rgb_parametry.jasnosc;


			//------LISTWY------------------------------
			if( !(   listwy_parametry.wl_wyl_listwy | (!glowne_parametry.wl_wyl_glowny)   ) )	dmx_buf[DMX_ADDRESS_LISTWY + 2] &= ~0x01;
			if(listwy_parametry.tryb_manual) dmx_buf[DMX_ADDRESS_LISTWY] = 0;
			else dmx_buf[DMX_ADDRESS_LISTWY] = listwy_parametry.tryb;



			//-----DYMIARKA--------------------

			if(dymiarka_parametry.en_dym && glowne_parametry.wl_wyl_glowny)  {
				dmx_buf[DMX_ADDRESS_DYMIARKA] = 200; //sprawdzic jaka wartosc
				flaga_timer_dym = 1;
				czas_zalaczenia_dymiarki_buf = dymiarka_parametry.czas_zalaczenia;
				TIMER3_1s_START;
				TCNT3 = 0;
			}
		}

		//-------PULSATOR------------------
		if(glowne_parametry.en_out || (flaga_wygaszenie_pulsator && (!glowne_parametry.wygaszenie)) ) {
			tryb_manual_pulsator_rgb_pomocniczy = pulsator_rgb_parametry.tryb_manual;
			if(pulsator_rgb_parametry.wl_wyl_pulsator && glowne_parametry.wl_wyl_glowny && (!glowne_parametry.wygaszenie)) {
				if(!pulsator_rgb_parametry.tryb_manual) {
					dmx_buf[DMX_ADDRESS_PULSATOR + 1] = 0;
					dmx_buf[DMX_ADDRESS_PULSATOR + 2] = 0;
					dmx_buf[DMX_ADDRESS_PULSATOR + 3] = 0;
					switch(pulsator_rgb_parametry.tryb) {
					case 1:
						dmx_buf[DMX_ADDRESS_PULSATOR + 4] = 45;
						break;
					case 2:
						dmx_buf[DMX_ADDRESS_PULSATOR + 4] = 101;
						break;
					case 3:
						dmx_buf[DMX_ADDRESS_PULSATOR + 4] = 129;
						break;
					case 4:
						dmx_buf[DMX_ADDRESS_PULSATOR + 4] = 157;
						break;
					case 5:
						dmx_buf[DMX_ADDRESS_PULSATOR + 4] = 185;
						break;
					case 6:
						dmx_buf[DMX_ADDRESS_PULSATOR + 4] = 230;
						break;
					case 7:
						dmx_buf[DMX_ADDRESS_PULSATOR + 4] = 254;
						break;
					}
				}
				else {
					dmx_buf[DMX_ADDRESS_PULSATOR + 4] = 0;
					if(pulsator_rgb_parametry.obrot) dmx_buf[DMX_ADDRESS_PULSATOR] = 200; //(wolno w lewo)
					else dmx_buf[DMX_ADDRESS_PULSATOR] = 0;
				}
			}
			//-------------------------
			flaga_wygaszenie_pulsator = 0;
			glowne_parametry.en_out = 0;
		}
	}
}

//DMX------------------------------------------------------------------------------------------------------

void usart0_init() {
	//usart0 na potrzeby DMX
	UBRR0H = 0;
	UBRR0L = 3;		   				//prdkosc 250kb/s dla taktowania 16MHz
	UCSR0A = 0;
	UCSR0B = (1<<TXEN0) | (1<<TXCIE0);			//zaczenie nadajnika i przerwa nadawczych
	UCSR0C = (3<<UCSZ00)|(1<<USBS0);			//format ramki 8n2, przy innym proc sprawdzic URSEL czy jest
}

void usart0_off() {
	UCSR0B = 0;
}

void send_dmx () {
	uint16_t j;
	for(j=1;j<DMX_SIZE;j++) dmx_to_send[j] = dmx_buf[j];
//	PORTB ^= (1<<PB5);
	usart0_off();
	DDR(DMX_TX_PORT) |= (1<<DMX_TX_PIN_NMB);		//pin dla TX jako wyjscie
	//BR
	PORT(DMX_TX_PORT) &= ~(1<<DMX_TX_PIN_NMB);
	_delay_us(120);
	//MAB
	PORT(DMX_TX_PORT) |= (1<<DMX_TX_PIN_NMB);
	_delay_us(12);
	usart0_init();
	UDR0 = dmx_to_send[ cnt_dmx = 0 ];		//start transmisji, ramka SC, reszta w przerwaniu
}

ISR(USART0_TX_vect) {
//	PORTB ^= (1<<PB5);
	cnt_dmx++;
	if(cnt_dmx < DMX_SIZE) {
		UDR0 = dmx_to_send[ cnt_dmx ];
	}
}

ISR(TIMER1_COMPA_vect) {
	flaga_40Hz = 1;
}

//--ODBIR-----BLUETOOTH--------------------------------------------------------------------------------

void usart1_init() {
	UBRR1H = 0;
	UBRR1L = 103;		   						//prdkosc 9.6 kb/s dla taktowania 16MHz
	UCSR1A = 0;
	UCSR1B = (1<<RXEN1) | (1<<RXCIE1);	//zaczenie odbiornika i przerwan odbiorczych
	UCSR1C = (3<<UCSZ10);			//format ramki 8n1
}

ISR(USART1_RX_vect)
{
	if(!(UCSR1A&(1<<FE1))) {
		if( cnt_uart1 == 0 ) {
			parametr = UDR1;
			TCNT0 = 0;
			TIMER0_5MS_START;
		}
		if( cnt_uart1 == 1) {
			argument = UDR1;
			flaga_odbioru_pakietu_blue = 1;
			TIMER0_5MS_STOP;
		}
		cnt_uart1++;
		if(cnt_uart1 == 2) cnt_uart1 = 0;
	}
}

ISR(TIMER0_COMP_vect) {
	TIMER0_5MS_STOP;
	cnt_uart1 = 0;
	flaga_odbioru_pakietu_blue = 0;
}

//dymiarka - odliczanie czasu

ISR(TIMER3_COMPA_vect) {
	static uint8_t cnt_timer3 = 0;
	cnt_timer3++;

	if(cnt_timer3 == czas_zalaczenia_dymiarki_buf)	{
		dmx_buf[DMX_ADDRESS_DYMIARKA] = 0;
		cnt_timer3 = 0;
		flaga_timer_dym = 0;
		TIMER3_1s_STOP;
	}
}